import requests


class Authenticator:
    DEV_ENDPOINT_URL = 'https://uaa.development.ine.com/uaa/authenticate'
    PRD_ENDPOINT_URL = 'https://uaa.ine.com/uaa/authenticate'

    @classmethod
    def get_token(cls, username, password, client, development=False):
        if development:
            url = cls.DEV_ENDPOINT_URL
        else:
            url = cls.PRD_ENDPOINT_URL

        response = requests.post(
            url,
            json={
                'username': username,
                'password': password,
                'client': client
            }
        )
        response.raise_for_status()

        try:
            auth_data = response.json()['data']['tokens']['data']['Bearer']
        except KeyError:
            raise RuntimeError('Wrong credentials.')

        return auth_data


from copy import deepcopy
from dataclasses import dataclass
from typing import Dict, List

import requests
from requests_toolbelt.multipart.encoder import MultipartEncoder
import colorful as cf

cf.use_256_ansi_colors()


@dataclass
class ProgrammingExerciseModel:
    exercise_uuid: str
    instructor_uuid: str
    name: str
    language: str
    description_md: str
    main_py: str
    run_py: str
    tests: Dict
    solutions: List[str]
    download_files: List[Dict]
    requirements: str

    EXECUTE_ENDPOINT_URL = 'https://execute.rmotr.com/execute'
    EXERCISE_ENDPOINT_URL = 'https://exercise.rmotr.com/api/v1/exercises'
    # EXERCISE_ENDPOINT_URL = 'http://localhost:8001/api/v1/exercises'
    # EXERCISE_ENDPOINT_URL = 'https://exercise.development.rmotr.com/api/v1/exercises'
    FILE_ENDPOINT_URL = 'https://file.rmotr.com/api/v1/files'
    # FILE_ENDPOINT_URL = 'http://localhost:8000/api/v1/files'
    # FILE_ENDPOINT_URL = 'https://file.development.rmotr.com/api/v1/files'

    def run(self):
        files = {'main.py': self.run_py}

        data = {
            "command": "python main.py",
            "files": files,
            "flavor": "ds-3.7",
            "language": "python",
            "produces": []
        }
        if self.download_files:
            data['download_files'] = self.download_files
        if self.requirements:
            data['pre_command'] = "pip install -q -r requirements.txt"
            data['files']['requirements.txt'] = self.requirements

        response = requests.post(self.EXECUTE_ENDPOINT_URL, json=data)
        if response.status_code != 200:
            print(response.json())
            return

        data = response.json()
        if data['successful']:
            print(cf.bold_green("Success!"))
            print(data['stdout'])
            return

        print(cf.bold_red("Error!\n\n"))
        print(cf.yellow(data['stderr']))
        print(data['stdout'])

    def test(self):

        files = deepcopy(self.tests)
        files['main.py'] = "".join(self.solutions)
        for _, value in self.tests.items():
            files['main.py'] += f"\n\n{value}"

        data = {
            "command": "py.test --json=report.json -q --tb=short main.py",
            "files": files,
            "flavor": "ds-3.7",
            "language": "python",
            "produces": [],
        }
        if self.download_files:
            data['download_files'] = self.download_files
        if self.requirements:
            data['pre_command'] = "pip install -q -r requirements.txt"
            data['files']['requirements.txt'] = self.requirements

        response = requests.post(self.EXECUTE_ENDPOINT_URL, json=data)
        if response.status_code != 200:
            print(response.json())
            return

        data = response.json()
        if data['successful']:
            print(cf.bold_green("Success!"))
            print(data['stdout'])
            return

        print(cf.bold_red("Error!\n\n"))
        print(cf.yellow(data['stderr']))
        print(data['stdout'])

    def create(self, auth_token):
        detail_url = self.EXERCISE_ENDPOINT_URL + f"/{self.exercise_uuid}"
        response = requests.get(
            detail_url,
            headers={
                'Authorization': f"Bearer {auth_token}"
            }
        )
        if response.status_code == requests.codes.ok:
            raise RuntimeError("Can't create an exercise if already exists.")

        data = {
            'id': self.exercise_uuid,
            'name': self.name,
            'description': self.description_md,
            'starter_code': self.main_py,
            'language': self.language,
            'instructor_id': self.instructor_uuid,
        }
        response = requests.post(
            self.EXERCISE_ENDPOINT_URL,
            headers={
                'Authorization': f"Bearer {auth_token}"
            },
            json=data
        )
        response.raise_for_status()
        data = response.json()
        print(cf.bold_green(f"Exercise created with id: {self.exercise_uuid}"))

        url = self.EXERCISE_ENDPOINT_URL + f"/{self.exercise_uuid}/tests"
        for key, value in self.tests.items():
            data = {
                'exercise': self.exercise_uuid,
                'name': key,
                'code': value
            }
            response = requests.post(
                url,
                headers={
                    'Authorization': f"Bearer {auth_token}"
                },
                json=data
            )
            response.raise_for_status()
            data = response.json()
            test_id = data.get('id')
            if not test_id:
                print(cf.bold_red(f"ERROR! Test could not be created."))
            else:
                print(cf.bold_green(f"Test created with id: {test_id}."))

        url = self.EXERCISE_ENDPOINT_URL + f"/{self.exercise_uuid}/solutions"
        solution_number = 1
        for solution in self.solutions:
            data = {
                'exercise': self.exercise_uuid,
                'name': f"Solution {solution_number}",
                'code': solution
            }
            solution_number += 1
            response = requests.post(
                url,
                headers={
                    'Authorization': f"Bearer {auth_token}"
                },
                json=data
            )
            response.raise_for_status()
            data = response.json()
            solution_id = data.get('id')
            if not solution_id:
                print(cf.bold_red(f"ERROR! Solution could not be created."))
            else:
                print(cf.bold_green(f"Solution created with id: {solution_id}."))

        url = self.FILE_ENDPOINT_URL + f"/upload"
        file_uuid_list = [] 
        for key, value in self.download_files.items():
            file = requests.get(value)
            data = MultipartEncoder(
                fields={
                    'name': key,
                    'type': key.split('.')[-1].lower(),
                    'file': (key, file.content),
                }
            )
            response = requests.post(
                url,
                headers={
                    'Authorization': f"Bearer {auth_token}",
                    'Content-Type': data.content_type,
                },
                data=data.to_string(),
            )
            if response.status_code == "400":
                print(response.json())
            else:
                response.raise_for_status()

            file_id = response.json().get('id')
            if not file_id:
                print(cf.bold_red(f"ERROR! File {key} could not be created."))
            else:
                print(cf.bold_green(f"File {key} created with id: {file_id}."))
                file_uuid_list.append(file_id)

        url = self.EXERCISE_ENDPOINT_URL + f"/{self.exercise_uuid}"
        response = requests.patch(
            url,
            headers={
                'Authorization': f"Bearer {auth_token}",
            },
            json={'files': file_uuid_list},
        )
        response.raise_for_status()
        print(cf.bold_green(f"Successfuly associated to exercise the file ids: {file_uuid_list}."))

    def submit(self):
        raise NotImplementedError()


import json
import requests

from .models import ProgrammingExerciseModel


class BaseStrategy:
    def is_valid(self, *args, **kwargs):
        return False

    def build_exercise(self, *args, **kwargs):
        raise NotImplementedError()


class GistStrategy(BaseStrategy):
    GITHUB_BASE_URL = "https://api.github.com/"

    @classmethod
    def endpoint(cls, *args):
        path = "/".join([arg.lstrip('/') for arg in args])
        return cls.GITHUB_BASE_URL + path

    @staticmethod
    def _read_gist_file(file_data):
        if not file_data['truncated']:
            return file_data['content']
        return requests.get(file_data['raw_url']).text

    def _get_file_content(self, data, file_name, required=False):
        if file_name not in data['files']:
            if required:
                raise ValueError(f"Invalid Gist: {file_name} not found")
            return ""
        return self._read_gist_file(data['files'][file_name])

    def _parse_tests(self, data):
        tests = {}
        for key, value in data['files'].items():
            if key.startswith('test_'):
                tests[key] = self._read_gist_file(value)

        if not tests:
            raise ValueError("Invalid Gist: tests not found")

        return tests

    def _parse_solutions(self, data):
        solutions = []
        for key, value in data['files'].items():
            if key.startswith('solution_'):
                solutions.append(self._read_gist_file(value))

        if not solutions:
            raise ValueError("Invalid Gist: solutions not found")

        return solutions

    def _parse_files(self, data):
        files = self._get_file_content(data, 'files.json', required=False)

        if files:
            return json.loads(files)

        return {}

    def _parse_manifest(self, data):
        manifest = json.loads(self._get_file_content(data, 'manifest.json', required=True))

        if not manifest.get('exercise_uuid'):
            raise RuntimeError("manifest.json: No value provided for 'exercise_uuid'.")
        elif not manifest.get('instructor_uuid'):
            raise RuntimeError("manifest.json: No value provided for 'instructor_uuid'.")
        elif not manifest.get('name'):
            raise RuntimeError("manifest.json: No value provided for 'name'.")

        return manifest

    ##################################################
    #             Public Interface                   #
    ##################################################
    @classmethod
    def is_valid(self, *args, **kwargs):
        return bool(kwargs.get('gist_id'))

    def build_exercise(self, *args, **kwargs):
        gist_id = kwargs['gist_id']
        resp = requests.get(self.endpoint('gists', gist_id))
        resp.raise_for_status()

        data = resp.json()
        manifest = self._parse_manifest(data)
        return ProgrammingExerciseModel(
            exercise_uuid=manifest['exercise_uuid'].strip(),
            instructor_uuid=manifest['instructor_uuid'].strip(),
            name=manifest['name'],
            language=manifest.get('language', 'python'),
            description_md=self._get_file_content(data, 'README.md'),
            main_py=self._get_file_content(data, 'main.py'),
            run_py=self._get_file_content(data, 'run.py'),
            solutions=self._parse_solutions(data),
            tests=self._parse_tests(data),
            download_files=self._parse_files(data),
            requirements=self._get_file_content(data, 'requirements.txt', required=False),
        )




# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['ine_exercises_tools']

package_data = \
{'': ['*']}

install_requires = \
['PyGithub>=1.51,<2.0',
 'click>=7.1.2,<8.0.0',
 'colorful>=0.5.4,<0.6.0',
 'requests-toolbelt>=0.9.1,<0.10.0']

setup_kwargs = {
    'name': 'ine-exercises-tools',
    'version': '0.0.2',
    'description': '',
    'long_description': None,
    'author': 'Santiago Basulto',
    'author_email': 'santiago.basulto@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
